@@ -1850,6 +1850,8 @@ UTESTCASE pass_begin_end(Utest *utest)
 	Autoref<TrieadOwner> ow1 = a1->makeTriead("t1");
 	Autoref<TrieadOwner> ow2 = a1->makeTriead("t2");
 	Autoref<TrieadOwner> ow3 = a1->makeTriead("t3");
+	Autoref<TrieadOwner> ow4 = a1->makeTriead("t4");
+	Autoref<TrieadOwner> ow5 = a1->makeTriead("t5");
 
 	// prepare fragments
 	RowType::FieldVec fld;
@@ -1870,6 +1872,10 @@ UTESTCASE pass_begin_end(Utest *utest)
 	Autoref<Unit> unit3 = ow3->unit();
 	Autoref<Unit::Tracer> trace3 = new Unit::StringNameTracer(false);
 	unit3->setTracer(trace3);
+	Autoref<Unit> unit4 = ow4->unit();
+	Autoref<Unit> unit5 = ow5->unit();
+	Autoref<Unit::Tracer> trace5 = new Unit::StringNameTracer(false);
+	unit5->setTracer(trace5);
 
 	// start with a writer
 	Autoref<Facet> fa1a = ow1->makeNexusWriter("nxa")
@@ -1887,7 +1893,7 @@ UTESTCASE pass_begin_end(Utest *utest)
 	ReaderQueue *far2a = FacetGuts::readerQueue(fa2a);
 	UT_ASSERT(far2a != NULL);
 
-	ow2->markReady(); // make the nexus visible for import
+	ow2->markReady();
 
 	// add a reader
 	Autoref<Facet> fa3a = ow3->importReader("t1", "nxa", "");
@@ -1900,15 +1906,42 @@ UTESTCASE pass_begin_end(Utest *utest)
 	Autoref<Label> lb3end = new DummyLabel(unit3, rt1, "end");
 	fa3a->getFnReturn()->getLabel("_END_")->chain(lb3end);
 
-	ow3->markReady(); // make the nexus visible for import
+	ow3->markReady();
+
+	// ow4 will be used to check that the _BEGIN_ and _END_
+	// also work properly in an imported nexus
+	Autoref<Facet> fa4a = ow4->importWriter("t1", "nxa", "");
+
+	ow4->markReady();
+
+	// ow5 is very much like ow3, only with chaining from a binding
+	// add a reader
+	Autoref<Facet> fa5a = ow5->importReader("t1", "nxa", "");
+	ReaderQueue *far5a = FacetGuts::readerQueue(fa5a);
+	UT_ASSERT(far5a != NULL);
+
+	// in t5 add the begin/end label chainings
+	Autoref<Label> lb5begin = new DummyLabel(unit5, rt1, "begin");
+	Autoref<Label> lb5end = new DummyLabel(unit5, rt1, "end");
+
+	Autoref<FnBinding> bind5 = FnBinding::make("bind5", fa5a->getFnReturn())
+		->addLabel("_BEGIN_", lb5begin, true)
+		->addLabel("_END_", lb5end, true);
+
+	fa5a->getFnReturn()->push(bind5);
+
+	ow5->markReady();
 
 	ow1->readyReady();
 	ow2->readyReady();
 	ow3->readyReady();
+	ow4->readyReady();
+	ow5->readyReady();
 
 	// ----------------------------------------------------------------------
 
 	UT_ASSERT(FnReturnGuts::isXtrayEmpty(fa1a->getFnReturn()));
+	UT_ASSERT(FnReturnGuts::isXtrayEmpty(fa4a->getFnReturn()));
 
 	// sending the BEGIN with no data and OP_INSERT puts nothing into the Xdata
 	
@@ -1917,6 +1950,11 @@ UTESTCASE pass_begin_end(Utest *utest)
 	UT_ASSERT(FnReturnGuts::isXtrayEmpty(fa1a->getFnReturn()));
 	UT_ASSERT(!ow2->nextXtray(false));
 
+	unit4->call(new Rowop(fa4a->getFnReturn()->getLabel("_BEGIN_"), 
+		Rowop::OP_INSERT, r0));
+	UT_ASSERT(FnReturnGuts::isXtrayEmpty(fa4a->getFnReturn()));
+	UT_ASSERT(!ow2->nextXtray(false));
+
 	// sending the END with no data and OP_INSERT puts nothing into the Xdata
 
 	unit1->call(new Rowop(fa1a->getFnReturn()->getLabel("_END_"), 
@@ -1924,6 +1962,11 @@ UTESTCASE pass_begin_end(Utest *utest)
 	UT_ASSERT(FnReturnGuts::isXtrayEmpty(fa1a->getFnReturn()));
 	UT_ASSERT(!ow2->nextXtray(false));
 
+	unit4->call(new Rowop(fa4a->getFnReturn()->getLabel("_END_"), 
+		Rowop::OP_INSERT, r0));
+	UT_ASSERT(FnReturnGuts::isXtrayEmpty(fa4a->getFnReturn()));
+	UT_ASSERT(!ow2->nextXtray(false));
+
 	// ----------------------------------------------------------------------
 
 	// send a row through without an explicit BEGIN/END
@@ -1957,42 +2000,88 @@ UTESTCASE pass_begin_end(Utest *utest)
 		if (UT_IS(tlog, expect)) printf("Expected: \"%s\"\n", expect.c_str());
 	}
 
+	// when the handlers are chained from a binding, begin/end get called
+	UT_ASSERT(ow5->nextXtray());
+	{
+		string tlog = trace5->getBuffer()->print();
+		trace5->clearBuffer();
+		string expect =
+			"unit 't5' before label 'nxa._BEGIN_' op OP_INSERT\n"
+			"unit 't5' before label 'begin' (chain 'nxa._BEGIN_') op OP_INSERT\n"
+			"unit 't5' before label 'nxa.one' op OP_INSERT\n"
+			"unit 't5' before label 'nxa._END_' op OP_INSERT\n"
+			"unit 't5' before label 'end' (chain 'nxa._END_') op OP_INSERT\n"
+		;
+		if (UT_IS(tlog, expect)) printf("Expected: \"%s\"\n", expect.c_str());
+	}
+
 	// ----------------------------------------------------------------------
 
 	// send a row through with the BEGIN/END that become implicit
 
+	UT_ASSERT(!ow2->nextXtray(false)); // _END_ will flush
+
 	unit1->call(new Rowop(fa1a->getFnReturn()->getLabel("_BEGIN_"), 
 		Rowop::OP_INSERT, r0));
 	unit1->call(new Rowop(fa1a->getFnReturn()->getLabel("one"), 
 		Rowop::OP_INSERT, r1));
+
 	unit1->call(new Rowop(fa1a->getFnReturn()->getLabel("_END_"), 
 		Rowop::OP_INSERT, r0));
 	// _END_ flushes the writer
 	UT_ASSERT(FnReturnGuts::isXtrayEmpty(fa1a->getFnReturn()));
 
-	UT_ASSERT(ow2->nextXtray());
-	{
-		string tlog = trace2->getBuffer()->print();
-		trace2->clearBuffer();
-		string expect =
-			"unit 't2' before label 'nxa.one' op OP_INSERT\n"
-		;
-		if (UT_IS(tlog, expect)) printf("Expected: \"%s\"\n", expect.c_str());
-	}
+	// same on ow4
+	unit4->call(new Rowop(fa4a->getFnReturn()->getLabel("_BEGIN_"), 
+		Rowop::OP_INSERT, r0));
+	unit4->call(new Rowop(fa4a->getFnReturn()->getLabel("one"), 
+		Rowop::OP_INSERT, r1));
 
-	// when the handlers are chained, begin/end get called
-	UT_ASSERT(ow3->nextXtray());
-	{
-		string tlog = trace3->getBuffer()->print();
-		trace3->clearBuffer();
-		string expect =
-			"unit 't3' before label 'nxa._BEGIN_' op OP_INSERT\n"
-			"unit 't3' before label 'begin' (chain 'nxa._BEGIN_') op OP_INSERT\n"
-			"unit 't3' before label 'nxa.one' op OP_INSERT\n"
-			"unit 't3' before label 'nxa._END_' op OP_INSERT\n"
-			"unit 't3' before label 'end' (chain 'nxa._END_') op OP_INSERT\n"
-		;
-		if (UT_IS(tlog, expect)) printf("Expected: \"%s\"\n", expect.c_str());
+	unit4->call(new Rowop(fa4a->getFnReturn()->getLabel("_END_"), 
+		Rowop::OP_INSERT, r0));
+	// _END_ flushes the writer
+	UT_ASSERT(FnReturnGuts::isXtrayEmpty(fa4a->getFnReturn()));
+
+	for (int i = 0; i < 2; i++) {
+		UT_ASSERT(ow2->nextXtray());
+		{
+			string tlog = trace2->getBuffer()->print();
+			trace2->clearBuffer();
+			string expect =
+				"unit 't2' before label 'nxa.one' op OP_INSERT\n"
+			;
+			if (UT_IS(tlog, expect)) printf("Pass %d Expected: \"%s\"\n", i, expect.c_str());
+		}
+
+		// when the handlers are chained, begin/end get called
+		UT_ASSERT(ow3->nextXtray());
+		{
+			string tlog = trace3->getBuffer()->print();
+			trace3->clearBuffer();
+			string expect =
+				"unit 't3' before label 'nxa._BEGIN_' op OP_INSERT\n"
+				"unit 't3' before label 'begin' (chain 'nxa._BEGIN_') op OP_INSERT\n"
+				"unit 't3' before label 'nxa.one' op OP_INSERT\n"
+				"unit 't3' before label 'nxa._END_' op OP_INSERT\n"
+				"unit 't3' before label 'end' (chain 'nxa._END_') op OP_INSERT\n"
+			;
+			if (UT_IS(tlog, expect)) printf("Pass %d Expected: \"%s\"\n", i, expect.c_str());
+		}
+
+		// when the handlers are chained from a binding, begin/end get called
+		UT_ASSERT(ow5->nextXtray());
+		{
+			string tlog = trace5->getBuffer()->print();
+			trace5->clearBuffer();
+			string expect =
+				"unit 't5' before label 'nxa._BEGIN_' op OP_INSERT\n"
+				"unit 't5' before label 'begin' (chain 'nxa._BEGIN_') op OP_INSERT\n"
+				"unit 't5' before label 'nxa.one' op OP_INSERT\n"
+				"unit 't5' before label 'nxa._END_' op OP_INSERT\n"
+				"unit 't5' before label 'end' (chain 'nxa._END_') op OP_INSERT\n"
+			;
+			if (UT_IS(tlog, expect)) printf("Pass %d Expected: \"%s\"\n", i, expect.c_str());
+		}
 	}
 
 	// ----------------------------------------------------------------------
@@ -2008,30 +2097,56 @@ UTESTCASE pass_begin_end(Utest *utest)
 	// _END_ flushes the writer
 	UT_ASSERT(FnReturnGuts::isXtrayEmpty(fa1a->getFnReturn()));
 
-	UT_ASSERT(ow2->nextXtray());
-	{
-		string tlog = trace2->getBuffer()->print();
-		trace2->clearBuffer();
-		string expect =
-			"unit 't2' before label 'nxa._BEGIN_' op OP_NOP\n"
-			"unit 't2' before label 'nxa.one' op OP_INSERT\n"
-			"unit 't2' before label 'nxa._END_' op OP_NOP\n"
-		;
-		if (UT_IS(tlog, expect)) printf("Expected: \"%s\"\n", expect.c_str());
-	}
+	// same on ow4
+	unit4->call(new Rowop(fa4a->getFnReturn()->getLabel("_BEGIN_"), 
+		Rowop::OP_NOP, r0));
+	unit4->call(new Rowop(fa4a->getFnReturn()->getLabel("one"), 
+		Rowop::OP_INSERT, r1));
+	unit4->call(new Rowop(fa4a->getFnReturn()->getLabel("_END_"), 
+		Rowop::OP_NOP, r0));
+	// _END_ flushes the writer
+	UT_ASSERT(FnReturnGuts::isXtrayEmpty(fa1a->getFnReturn()));
 
-	UT_ASSERT(ow3->nextXtray());
-	{
-		string tlog = trace3->getBuffer()->print();
-		trace3->clearBuffer();
-		string expect =
-			"unit 't3' before label 'nxa._BEGIN_' op OP_NOP\n"
-			"unit 't3' before label 'begin' (chain 'nxa._BEGIN_') op OP_NOP\n"
-			"unit 't3' before label 'nxa.one' op OP_INSERT\n"
-			"unit 't3' before label 'nxa._END_' op OP_NOP\n"
-			"unit 't3' before label 'end' (chain 'nxa._END_') op OP_NOP\n"
-		;
-		if (UT_IS(tlog, expect)) printf("Expected: \"%s\"\n", expect.c_str());
+	for (int i = 0; i < 2; i++) {
+		UT_ASSERT(ow2->nextXtray());
+		{
+			string tlog = trace2->getBuffer()->print();
+			trace2->clearBuffer();
+			string expect =
+				"unit 't2' before label 'nxa._BEGIN_' op OP_NOP\n"
+				"unit 't2' before label 'nxa.one' op OP_INSERT\n"
+				"unit 't2' before label 'nxa._END_' op OP_NOP\n"
+			;
+			if (UT_IS(tlog, expect)) printf("Pass %d Expected: \"%s\"\n", i, expect.c_str());
+		}
+
+		UT_ASSERT(ow3->nextXtray());
+		{
+			string tlog = trace3->getBuffer()->print();
+			trace3->clearBuffer();
+			string expect =
+				"unit 't3' before label 'nxa._BEGIN_' op OP_NOP\n"
+				"unit 't3' before label 'begin' (chain 'nxa._BEGIN_') op OP_NOP\n"
+				"unit 't3' before label 'nxa.one' op OP_INSERT\n"
+				"unit 't3' before label 'nxa._END_' op OP_NOP\n"
+				"unit 't3' before label 'end' (chain 'nxa._END_') op OP_NOP\n"
+			;
+			if (UT_IS(tlog, expect)) printf("Pass %d Expected: \"%s\"\n", i, expect.c_str());
+		}
+
+		UT_ASSERT(ow5->nextXtray());
+		{
+			string tlog = trace5->getBuffer()->print();
+			trace5->clearBuffer();
+			string expect =
+				"unit 't5' before label 'nxa._BEGIN_' op OP_NOP\n"
+				"unit 't5' before label 'begin' (chain 'nxa._BEGIN_') op OP_NOP\n"
+				"unit 't5' before label 'nxa.one' op OP_INSERT\n"
+				"unit 't5' before label 'nxa._END_' op OP_NOP\n"
+				"unit 't5' before label 'end' (chain 'nxa._END_') op OP_NOP\n"
+			;
+			if (UT_IS(tlog, expect)) printf("Pass %d Expected: \"%s\"\n", i, expect.c_str());
+		}
 	}
 
 	// ----------------------------------------------------------------------
@@ -2047,30 +2162,56 @@ UTESTCASE pass_begin_end(Utest *utest)
 	// _END_ flushes the writer
 	UT_ASSERT(FnReturnGuts::isXtrayEmpty(fa1a->getFnReturn()));
 
-	UT_ASSERT(ow2->nextXtray());
-	{
-		string tlog = trace2->getBuffer()->print();
-		trace2->clearBuffer();
-		string expect =
-			"unit 't2' before label 'nxa._BEGIN_' op OP_INSERT\n"
-			"unit 't2' before label 'nxa.one' op OP_INSERT\n"
-			"unit 't2' before label 'nxa._END_' op OP_INSERT\n"
-		;
-		if (UT_IS(tlog, expect)) printf("Expected: \"%s\"\n", expect.c_str());
-	}
+	// same on ow4
+	unit4->call(new Rowop(fa4a->getFnReturn()->getLabel("_BEGIN_"), 
+		Rowop::OP_INSERT, r1));
+	unit4->call(new Rowop(fa4a->getFnReturn()->getLabel("one"), 
+		Rowop::OP_INSERT, r1));
+	unit4->call(new Rowop(fa4a->getFnReturn()->getLabel("_END_"), 
+		Rowop::OP_INSERT, r1));
+	// _END_ flushes the writer
+	UT_ASSERT(FnReturnGuts::isXtrayEmpty(fa4a->getFnReturn()));
 
-	UT_ASSERT(ow3->nextXtray());
-	{
-		string tlog = trace3->getBuffer()->print();
-		trace3->clearBuffer();
-		string expect =
-			"unit 't3' before label 'nxa._BEGIN_' op OP_INSERT\n"
-			"unit 't3' before label 'begin' (chain 'nxa._BEGIN_') op OP_INSERT\n"
-			"unit 't3' before label 'nxa.one' op OP_INSERT\n"
-			"unit 't3' before label 'nxa._END_' op OP_INSERT\n"
-			"unit 't3' before label 'end' (chain 'nxa._END_') op OP_INSERT\n"
-		;
-		if (UT_IS(tlog, expect)) printf("Expected: \"%s\"\n", expect.c_str());
+	for (int i = 0; i < 2; i++) {
+		UT_ASSERT(ow2->nextXtray());
+		{
+			string tlog = trace2->getBuffer()->print();
+			trace2->clearBuffer();
+			string expect =
+				"unit 't2' before label 'nxa._BEGIN_' op OP_INSERT\n"
+				"unit 't2' before label 'nxa.one' op OP_INSERT\n"
+				"unit 't2' before label 'nxa._END_' op OP_INSERT\n"
+			;
+			if (UT_IS(tlog, expect)) printf("Pass %d Expected: \"%s\"\n", i, expect.c_str());
+		}
+
+		UT_ASSERT(ow3->nextXtray());
+		{
+			string tlog = trace3->getBuffer()->print();
+			trace3->clearBuffer();
+			string expect =
+				"unit 't3' before label 'nxa._BEGIN_' op OP_INSERT\n"
+				"unit 't3' before label 'begin' (chain 'nxa._BEGIN_') op OP_INSERT\n"
+				"unit 't3' before label 'nxa.one' op OP_INSERT\n"
+				"unit 't3' before label 'nxa._END_' op OP_INSERT\n"
+				"unit 't3' before label 'end' (chain 'nxa._END_') op OP_INSERT\n"
+			;
+			if (UT_IS(tlog, expect)) printf("Pass %d Expected: \"%s\"\n", i, expect.c_str());
+		}
+
+		UT_ASSERT(ow5->nextXtray());
+		{
+			string tlog = trace5->getBuffer()->print();
+			trace5->clearBuffer();
+			string expect =
+				"unit 't5' before label 'nxa._BEGIN_' op OP_INSERT\n"
+				"unit 't5' before label 'begin' (chain 'nxa._BEGIN_') op OP_INSERT\n"
+				"unit 't5' before label 'nxa.one' op OP_INSERT\n"
+				"unit 't5' before label 'nxa._END_' op OP_INSERT\n"
+				"unit 't5' before label 'end' (chain 'nxa._END_') op OP_INSERT\n"
+			;
+			if (UT_IS(tlog, expect)) printf("Pass %d Expected: \"%s\"\n", i, expect.c_str());
+		}
 	}
 
 	// ----------------------------------------------------------------------
@@ -2082,27 +2223,48 @@ UTESTCASE pass_begin_end(Utest *utest)
 	// _END_ flushes the writer
 	UT_ASSERT(FnReturnGuts::isXtrayEmpty(fa1a->getFnReturn()));
 
-	UT_ASSERT(ow2->nextXtray());
-	{
-		string tlog = trace2->getBuffer()->print();
-		trace2->clearBuffer();
-		string expect =
-			"unit 't2' before label 'nxa._END_' op OP_INSERT\n"
-		;
-		if (UT_IS(tlog, expect)) printf("Expected: \"%s\"\n", expect.c_str());
-	}
+	// same on ow4
+	unit4->call(new Rowop(fa4a->getFnReturn()->getLabel("_END_"), 
+		Rowop::OP_INSERT, r1));
+	// _END_ flushes the writer
+	UT_ASSERT(FnReturnGuts::isXtrayEmpty(fa4a->getFnReturn()));
 
-	UT_ASSERT(ow3->nextXtray());
-	{
-		string tlog = trace3->getBuffer()->print();
-		trace3->clearBuffer();
-		string expect =
-			"unit 't3' before label 'nxa._BEGIN_' op OP_INSERT\n"
-			"unit 't3' before label 'begin' (chain 'nxa._BEGIN_') op OP_INSERT\n"
-			"unit 't3' before label 'nxa._END_' op OP_INSERT\n"
-			"unit 't3' before label 'end' (chain 'nxa._END_') op OP_INSERT\n"
-		;
-		if (UT_IS(tlog, expect)) printf("Expected: \"%s\"\n", expect.c_str());
+	for (int i = 0; i < 2; i++) {
+		UT_ASSERT(ow2->nextXtray());
+		{
+			string tlog = trace2->getBuffer()->print();
+			trace2->clearBuffer();
+			string expect =
+				"unit 't2' before label 'nxa._END_' op OP_INSERT\n"
+			;
+			if (UT_IS(tlog, expect)) printf("Pass %d Expected: \"%s\"\n", i, expect.c_str());
+		}
+
+		UT_ASSERT(ow3->nextXtray());
+		{
+			string tlog = trace3->getBuffer()->print();
+			trace3->clearBuffer();
+			string expect =
+				"unit 't3' before label 'nxa._BEGIN_' op OP_INSERT\n"
+				"unit 't3' before label 'begin' (chain 'nxa._BEGIN_') op OP_INSERT\n"
+				"unit 't3' before label 'nxa._END_' op OP_INSERT\n"
+				"unit 't3' before label 'end' (chain 'nxa._END_') op OP_INSERT\n"
+			;
+			if (UT_IS(tlog, expect)) printf("Pass %d Expected: \"%s\"\n", i, expect.c_str());
+		}
+
+		UT_ASSERT(ow5->nextXtray());
+		{
+			string tlog = trace5->getBuffer()->print();
+			trace5->clearBuffer();
+			string expect =
+				"unit 't5' before label 'nxa._BEGIN_' op OP_INSERT\n"
+				"unit 't5' before label 'begin' (chain 'nxa._BEGIN_') op OP_INSERT\n"
+				"unit 't5' before label 'nxa._END_' op OP_INSERT\n"
+				"unit 't5' before label 'end' (chain 'nxa._END_') op OP_INSERT\n"
+			;
+			if (UT_IS(tlog, expect)) printf("Pass %d Expected: \"%s\"\n", i, expect.c_str());
+		}
 	}
 
 	// ----------------------------------------------------------------------
@@ -2111,6 +2273,8 @@ UTESTCASE pass_begin_end(Utest *utest)
 	ow1->markDead();
 	ow2->markDead();
 	ow3->markDead();
+	ow4->markDead();
+	ow5->markDead();
 	a1->harvester();
 
 	restore_uncatchable();
