@@ -72,6 +72,7 @@ public:
 class FnReturn: public Starget
 {
 	friend class Facet;
+	friend class TrieadOwner;
 protected:
 	// The class of labels created inside FnReturn, that forward the rowops
 	// to the final destination.
@@ -364,6 +365,18 @@ protected:
 
 	// }
 
+	// Interface for TrieadOwner (closely connected to Facet).
+	// {
+
+	// Check whether the label either has a chaining directly from the
+	// FnReturn or is defined in the current pushed binding. This is used by
+	// TrieadOwner to decide whether it needs to generate the artificial
+	// _BEGIN_ and _END_ rowops.
+	// @param idx - index of the label
+	// @return - pointer to the label of there is a chaining, NULL if not
+	Label *checkLabelChained(int idx) const;
+	// }
+
 	Unit *unit_; // not a reference, used only to create the labels
 	Facet *facet_; // not a reference, the facet that wraps this object, or NULL; set directly by Facet
 	string name_; // human-readable name, and base for the label names
