@@ -1,5 +1,5 @@
 //
-// (C) Copyright 2011-2012 Sergey A. Babkin.
+// (C) Copyright 2011-2013 Sergey A. Babkin.
 // This file is a part of Triceps.
 // See the file COPYRIGHT for the copyright notice and license information
 //
@@ -21,8 +21,16 @@ FifoIndexType::FifoIndexType(size_t limit, bool jumping, bool reverse) :
 { 
 }
 
-FifoIndexType::FifoIndexType(const FifoIndexType &orig) :
-	IndexType(orig),
+FifoIndexType::FifoIndexType(const FifoIndexType &orig, bool flat) :
+	IndexType(orig, flat),
+	limit_(orig.limit_),
+	jumping_(orig.jumping_),
+	reverse_(orig.reverse_)
+{
+}
+
+FifoIndexType::FifoIndexType(const FifoIndexType &orig, HoldRowTypes *holder) :
+	IndexType(orig, holder),
 	limit_(orig.limit_),
 	jumping_(orig.jumping_),
 	reverse_(orig.reverse_)
@@ -89,9 +97,14 @@ void FifoIndexType::printTo(string &res, const string &indent, const string &sub
 	printSubelementsTo(res, indent, subindent);
 }
 
-IndexType *FifoIndexType::copy() const
+IndexType *FifoIndexType::copy(bool flat) const
 {
-	return new FifoIndexType(*this);
+	return new FifoIndexType(*this, flat);
+}
+
+IndexType *FifoIndexType::deepCopy(HoldRowTypes *holder) const
+{
+	return new FifoIndexType(*this, holder);
 }
 
 void FifoIndexType::initialize()
