@@ -1,5 +1,5 @@
 //
-// (C) Copyright 2011-2012 Sergey A. Babkin.
+// (C) Copyright 2011-2013 Sergey A. Babkin.
 // This file is a part of Triceps.
 // See the file COPYRIGHT for the copyright notice and license information
 //
@@ -25,6 +25,7 @@ class GroupHandleType;
 class Index;
 class Table;
 class Aggregator;
+class HoldRowTypes;
 
 // connection of indexes into a tree
 class  IndexTypeRef 
@@ -49,8 +50,13 @@ public:
 	
 	IndexTypeVec();
 	IndexTypeVec(size_t size);
-	// Populate with the copy of the original types
-	IndexTypeVec(const IndexTypeVec &orig);
+	// Populate with the copy of the original types.
+	// @param flat - flag: do not include any nested indexes, which pretty
+	//        much makes this constructor the same as the default one
+	IndexTypeVec(const IndexTypeVec &orig, bool flat);
+	// Populate with the copy of the original types, and preserve the
+	// sharedness of RowType references.
+	IndexTypeVec(const IndexTypeVec &orig, HoldRowTypes *holder);
 
 	// Find the nested index by name.
 	// @param name - name of the nested index
@@ -93,6 +99,7 @@ public:
 	void clearRowHandle(RowHandle *rh) const;
 
 private:
+	IndexTypeVec(const IndexTypeVec &orig);
 	void operator=(const IndexTypeVec &);
 };
 
@@ -197,7 +204,35 @@ public:
 	// Make a copy of this type. The copy is always uninitialized, no
 	// matter whther it was made from an initialized one or not.
 	// The subclasses must define the actual copying.
-	virtual IndexType *copy() const = 0;
+	//
+	// The flat copying allows to get rid of any structure around,
+	// and do such things as rebuilding a table type with a subset of
+	// the original indexes.
+	//
+	// @param flat - flag: copy just this index, no nested contents (indexes,
+	//        aggregators etc.)
+	virtual IndexType *copy(bool flat = false) const = 0;
+	// Make a deep copy of this type. The copy is always uninitialized, no
+	// matter whther it was made from an initialized one or not.
+	// The subclasses must define the actual copying.
+	// Preserve the sharedness of RowType references in the copy.
+	//
+	// Note that using this method holder==NULL has a different meaning 
+	// than copy(): copy() will keep the references to the original
+	// RowType while deepCopy(NULL) will make an independent copy
+	// of for each use of RowType.
+	//
+	// Here there is no use in the holder having the default of NULL
+	// because it would produce something seriously undesirable.
+	// Just use copy() instead if you don't need the deepness.
+	//
+	// @param holder - helper object that makes sure that multiple
+	//        references to the same row type stay multiple references
+	//        to the same copied row type, not multiple row types
+	//        (unless it's NULL, which reverts to plain copying).
+	//        The caller has to keep a reference to the holder for
+	//        the duration.
+	virtual IndexType *deepCopy(HoldRowTypes *holder) const = 0;
 
 	// @return - true if there are no nested indexes
 	bool isLeaf() const
@@ -282,7 +317,12 @@ protected:
 
 	// can be constructed only from subclasses
 	IndexType(IndexId it);
-	IndexType(const IndexType &orig); 
+	// Copy.
+	// @param flat - flag: copy just this index, no nested contents (indexes,
+	//        aggregators etc.)
+	IndexType(const IndexType &orig, bool flat); 
+	// Copy and preserve the sharedness of RowType references in the copy.
+	IndexType(const IndexType &orig, HoldRowTypes *holder); 
 
 	// let the index find itself in parent and table type
 	// @param tabtype - table type where this index type belongs (to only one table type!)
@@ -607,6 +647,7 @@ protected:
 
 private:
 	IndexType();
+	IndexType(const IndexType &orig); 
 	void operator=(const IndexType &);
 };
 
