@@ -1,5 +1,5 @@
 //
-// (C) Copyright 2011-2012 Sergey A. Babkin.
+// (C) Copyright 2011-2013 Sergey A. Babkin.
 // This file is a part of Triceps.
 // See the file COPYRIGHT for the copyright notice and license information
 //
@@ -24,7 +24,8 @@ public:
 
 	// from IndexType
 	virtual const_Onceref<NameSet> getKey() const;
-	virtual IndexType *copy() const;
+	virtual IndexType *copy(bool flat = false) const;
+	virtual IndexType *deepCopy(HoldRowTypes *holder) const;
 	virtual void initialize();
 	virtual Index *makeIndex(const TableType *tabtype, Table *table) const;
 	virtual void initRowHandleSection(RowHandle *rh) const;
@@ -38,7 +39,9 @@ protected:
 	RootIndexType();
 
 	// used by copy()
-	RootIndexType(const RootIndexType &orig);
+	RootIndexType(const RootIndexType &orig, bool flat);
+	// used by deepCopy()
+	RootIndexType(const RootIndexType &orig, HoldRowTypes *holder);
 };
 
 }; // TRICEPS_NS
