@@ -1,5 +1,5 @@
 //
-// (C) Copyright 2011-2012 Sergey A. Babkin.
+// (C) Copyright 2011-2013 Sergey A. Babkin.
 // This file is a part of Triceps.
 // See the file COPYRIGHT for the copyright notice and license information
 //
@@ -14,12 +14,19 @@
 #include <sched/FnReturn.h>
 #include <table/Table.h>
 #include <mem/Rhref.h>
+#include <app/App.h>
+#include <app/AutoDrain.h>
 
 namespace TRICEPS_NS {
 
 // for extra safety, add a magic in front of each wrapper
 
 struct WrapMagic {
+	~WrapMagic()
+	{
+		(*(int64_t *)v_) = 0; // makes sure that it gets invalidated
+	}
+
 	char v_[8]; // 8 bytes to make a single 64-bit comparison
 
 	bool operator!=(const WrapMagic &wm) const
@@ -198,6 +205,13 @@ DEFINE_WRAP(Table);
 DEFINE_WRAP(Index);
 DEFINE_WRAP2(Table, Rhref, RowHandle);
 
+DEFINE_WRAP(App);
+DEFINE_WRAP(Triead);
+DEFINE_WRAP(TrieadOwner);
+DEFINE_WRAP(Facet);
+DEFINE_WRAP(Nexus);
+DEFINE_WRAP(AutoDrain);
+
 #undef DEFINE_WRAP
 #undef DEFINE_WRAP2
 #undef DEFINE_WRAP_IDENT
